// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.35.2
// 	protoc        v3.11.4
// source: github.com/moby/buildkit/session/sshforward/ssh.proto

package sshforward

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// BytesMessage contains a chunk of byte data
type BytesMessage struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Data []byte `protobuf:"bytes,1,opt,name=data,proto3" json:"data,omitempty"`
}

func (x *BytesMessage) Reset() {
	*x = BytesMessage{}
	mi := &file_github_com_moby_buildkit_session_sshforward_ssh_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BytesMessage) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BytesMessage) ProtoMessage() {}

func (x *BytesMessage) ProtoReflect() protoreflect.Message {
	mi := &file_github_com_moby_buildkit_session_sshforward_ssh_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BytesMessage.ProtoReflect.Descriptor instead.
func (*BytesMessage) Descriptor() ([]byte, []int) {
	return file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDescGZIP(), []int{0}
}

func (x *BytesMessage) GetData() []byte {
	if x != nil {
		return x.Data
	}
	return nil
}

type CheckAgentRequest struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	ID string `protobuf:"bytes,1,opt,name=ID,proto3" json:"ID,omitempty"`
}

func (x *CheckAgentRequest) Reset() {
	*x = CheckAgentRequest{}
	mi := &file_github_com_moby_buildkit_session_sshforward_ssh_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CheckAgentRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CheckAgentRequest) ProtoMessage() {}

func (x *CheckAgentRequest) ProtoReflect() protoreflect.Message {
	mi := &file_github_com_moby_buildkit_session_sshforward_ssh_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CheckAgentRequest.ProtoReflect.Descriptor instead.
func (*CheckAgentRequest) Descriptor() ([]byte, []int) {
	return file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDescGZIP(), []int{1}
}

func (x *CheckAgentRequest) GetID() string {
	if x != nil {
		return x.ID
	}
	return ""
}

type CheckAgentResponse struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields
}

func (x *CheckAgentResponse) Reset() {
	*x = CheckAgentResponse{}
	mi := &file_github_com_moby_buildkit_session_sshforward_ssh_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CheckAgentResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CheckAgentResponse) ProtoMessage() {}

func (x *CheckAgentResponse) ProtoReflect() protoreflect.Message {
	mi := &file_github_com_moby_buildkit_session_sshforward_ssh_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CheckAgentResponse.ProtoReflect.Descriptor instead.
func (*CheckAgentResponse) Descriptor() ([]byte, []int) {
	return file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDescGZIP(), []int{2}
}

var File_github_com_moby_buildkit_session_sshforward_ssh_proto protoreflect.FileDescriptor

var file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDesc = []byte{
	0x0a, 0x35, 0x67, 0x69, 0x74, 0x68, 0x75, 0x62, 0x2e, 0x63, 0x6f, 0x6d, 0x2f, 0x6d, 0x6f, 0x62,
	0x79, 0x2f, 0x62, 0x75, 0x69, 0x6c, 0x64, 0x6b, 0x69, 0x74, 0x2f, 0x73, 0x65, 0x73, 0x73, 0x69,
	0x6f, 0x6e, 0x2f, 0x73, 0x73, 0x68, 0x66, 0x6f, 0x72, 0x77, 0x61, 0x72, 0x64, 0x2f, 0x73, 0x73,
	0x68, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x12, 0x12, 0x6d, 0x6f, 0x62, 0x79, 0x2e, 0x73, 0x73,
	0x68, 0x66, 0x6f, 0x72, 0x77, 0x61, 0x72, 0x64, 0x2e, 0x76, 0x31, 0x22, 0x22, 0x0a, 0x0c, 0x42,
	0x79, 0x74, 0x65, 0x73, 0x4d, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x12, 0x12, 0x0a, 0x04, 0x64,
	0x61, 0x74, 0x61, 0x18, 0x01, 0x20, 0x01, 0x28, 0x0c, 0x52, 0x04, 0x64, 0x61, 0x74, 0x61, 0x22,
	0x23, 0x0a, 0x11, 0x43, 0x68, 0x65, 0x63, 0x6b, 0x41, 0x67, 0x65, 0x6e, 0x74, 0x52, 0x65, 0x71,
	0x75, 0x65, 0x73, 0x74, 0x12, 0x0e, 0x0a, 0x02, 0x49, 0x44, 0x18, 0x01, 0x20, 0x01, 0x28, 0x09,
	0x52, 0x02, 0x49, 0x44, 0x22, 0x14, 0x0a, 0x12, 0x43, 0x68, 0x65, 0x63, 0x6b, 0x41, 0x67, 0x65,
	0x6e, 0x74, 0x52, 0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x32, 0xba, 0x01, 0x0a, 0x03, 0x53,
	0x53, 0x48, 0x12, 0x5b, 0x0a, 0x0a, 0x43, 0x68, 0x65, 0x63, 0x6b, 0x41, 0x67, 0x65, 0x6e, 0x74,
	0x12, 0x25, 0x2e, 0x6d, 0x6f, 0x62, 0x79, 0x2e, 0x73, 0x73, 0x68, 0x66, 0x6f, 0x72, 0x77, 0x61,
	0x72, 0x64, 0x2e, 0x76, 0x31, 0x2e, 0x43, 0x68, 0x65, 0x63, 0x6b, 0x41, 0x67, 0x65, 0x6e, 0x74,
	0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x1a, 0x26, 0x2e, 0x6d, 0x6f, 0x62, 0x79, 0x2e, 0x73,
	0x73, 0x68, 0x66, 0x6f, 0x72, 0x77, 0x61, 0x72, 0x64, 0x2e, 0x76, 0x31, 0x2e, 0x43, 0x68, 0x65,
	0x63, 0x6b, 0x41, 0x67, 0x65, 0x6e, 0x74, 0x52, 0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x12,
	0x56, 0x0a, 0x0c, 0x46, 0x6f, 0x72, 0x77, 0x61, 0x72, 0x64, 0x41, 0x67, 0x65, 0x6e, 0x74, 0x12,
	0x20, 0x2e, 0x6d, 0x6f, 0x62, 0x79, 0x2e, 0x73, 0x73, 0x68, 0x66, 0x6f, 0x72, 0x77, 0x61, 0x72,
	0x64, 0x2e, 0x76, 0x31, 0x2e, 0x42, 0x79, 0x74, 0x65, 0x73, 0x4d, 0x65, 0x73, 0x73, 0x61, 0x67,
	0x65, 0x1a, 0x20, 0x2e, 0x6d, 0x6f, 0x62, 0x79, 0x2e, 0x73, 0x73, 0x68, 0x66, 0x6f, 0x72, 0x77,
	0x61, 0x72, 0x64, 0x2e, 0x76, 0x31, 0x2e, 0x42, 0x79, 0x74, 0x65, 0x73, 0x4d, 0x65, 0x73, 0x73,
	0x61, 0x67, 0x65, 0x28, 0x01, 0x30, 0x01, 0x42, 0x2d, 0x5a, 0x2b, 0x67, 0x69, 0x74, 0x68, 0x75,
	0x62, 0x2e, 0x63, 0x6f, 0x6d, 0x2f, 0x6d, 0x6f, 0x62, 0x79, 0x2f, 0x62, 0x75, 0x69, 0x6c, 0x64,
	0x6b, 0x69, 0x74, 0x2f, 0x73, 0x65, 0x73, 0x73, 0x69, 0x6f, 0x6e, 0x2f, 0x73, 0x73, 0x68, 0x66,
	0x6f, 0x72, 0x77, 0x61, 0x72, 0x64, 0x62, 0x06, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x33,
}

var (
	file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDescOnce sync.Once
	file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDescData = file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDesc
)

func file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDescGZIP() []byte {
	file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDescOnce.Do(func() {
		file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDescData = protoimpl.X.CompressGZIP(file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDescData)
	})
	return file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDescData
}

var file_github_com_moby_buildkit_session_sshforward_ssh_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_github_com_moby_buildkit_session_sshforward_ssh_proto_goTypes = []any{
	(*BytesMessage)(nil),       // 0: moby.sshforward.v1.BytesMessage
	(*CheckAgentRequest)(nil),  // 1: moby.sshforward.v1.CheckAgentRequest
	(*CheckAgentResponse)(nil), // 2: moby.sshforward.v1.CheckAgentResponse
}
var file_github_com_moby_buildkit_session_sshforward_ssh_proto_depIdxs = []int32{
	1, // 0: moby.sshforward.v1.SSH.CheckAgent:input_type -> moby.sshforward.v1.CheckAgentRequest
	0, // 1: moby.sshforward.v1.SSH.ForwardAgent:input_type -> moby.sshforward.v1.BytesMessage
	2, // 2: moby.sshforward.v1.SSH.CheckAgent:output_type -> moby.sshforward.v1.CheckAgentResponse
	0, // 3: moby.sshforward.v1.SSH.ForwardAgent:output_type -> moby.sshforward.v1.BytesMessage
	2, // [2:4] is the sub-list for method output_type
	0, // [0:2] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_github_com_moby_buildkit_session_sshforward_ssh_proto_init() }
func file_github_com_moby_buildkit_session_sshforward_ssh_proto_init() {
	if File_github_com_moby_buildkit_session_sshforward_ssh_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDesc,
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_github_com_moby_buildkit_session_sshforward_ssh_proto_goTypes,
		DependencyIndexes: file_github_com_moby_buildkit_session_sshforward_ssh_proto_depIdxs,
		MessageInfos:      file_github_com_moby_buildkit_session_sshforward_ssh_proto_msgTypes,
	}.Build()
	File_github_com_moby_buildkit_session_sshforward_ssh_proto = out.File
	file_github_com_moby_buildkit_session_sshforward_ssh_proto_rawDesc = nil
	file_github_com_moby_buildkit_session_sshforward_ssh_proto_goTypes = nil
	file_github_com_moby_buildkit_session_sshforward_ssh_proto_depIdxs = nil
}
