// Code generated by "stringer -type=InstrumentKind -trimprefix=InstrumentKind"; DO NOT EDIT.

package metric

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[instrumentKindUndefined-0]
	_ = x[InstrumentKindCounter-1]
	_ = x[InstrumentKindUpDownCounter-2]
	_ = x[InstrumentKindHistogram-3]
	_ = x[InstrumentKindObservableCounter-4]
	_ = x[InstrumentKindObservableUpDownCounter-5]
	_ = x[InstrumentKindObservableGauge-6]
	_ = x[InstrumentKindGauge-7]
}

const _InstrumentKind_name = "instrumentKindUndefinedCounterUpDownCounterHistogramObservableCounterObservableUpDownCounterObservableGaugeGauge"

var _InstrumentKind_index = [...]uint8{0, 23, 30, 43, 52, 69, 92, 107, 112}

func (i InstrumentKind) String() string {
	if i >= InstrumentKind(len(_InstrumentKind_index)-1) {
		return "InstrumentKind(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _InstrumentKind_name[_InstrumentKind_index[i]:_InstrumentKind_index[i+1]]
}
