/*
Copyright 2023 The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package remotecommand

// streamProtocolV5 add support for V5 of the remote command subprotocol.
// For the streamProtocolHandler, this version is the same as V4.
type streamProtocolV5 struct {
	*streamProtocolV4
}

var _ streamProtocolHandler = &streamProtocolV5{}

func newStreamProtocolV5(options StreamOptions) streamProtocolHandler {
	return &streamProtocolV5{
		streamProtocolV4: newStreamProtocolV4(options).(*streamProtocolV4),
	}
}

func (p *streamProtocolV5) stream(conn streamCreator) error {
	return p.streamProtocolV4.stream(conn)
}
